;-----------------------------------------------------------------------------
;
; Mini Cookie
;       by Sebastian Mihai, 2015
;       http://sebastianmihai.com
;
; A small tech demo of a bear who can be guided to eat all on-screen cookies.
;
;-----------------------------------------------------------------------------

	.include includes/header.asm
	
;-----------------------------------------------------------------------------
; Variables
;-----------------------------------------------------------------------------
.org 0x1300

bearX: .ds 1
bearY: .ds 1

gameBoard: .ds 9 0
	
;-----------------------------------------------------------------------------
; Header
;-----------------------------------------------------------------------------

	.org 0x2100
	.db		"MN"			; cam be anything except 0xBF,0xD9
	jmp		start

	.org 0x2108
	; interrupt handlers come here


	.org	0x21A4
	.db		"NINTENDO"		; magic string
	.db		"__________"	; name
	.db		"____"			; 4-character gamecode
	.db		"__"			; ?
	
start:
	movw nn, IO_BEGIN
	; set tiled mode 12x8, non-inverted, and enable video
	movb [nn+VIDEO_0], $02+$08
	movb [nn+VIDEO_1], $01+$08

	; set tile data pointer point to the loaded bitmaps
	mov x1, tile_data
	movw [NN+VIDEO_TILEDATA], x1
	
	mov a, 1
	mov [bearX], a
	mov [bearY], a

	call fill_screen
	
main_loop:
	mov	x1, tile_bear
	mov a, [bearX]
	mov b, [bearY]
	call put_tile

	call keys_wait_for_release
	call keys_wait_for_press

check_input:
	test [nn+BUTTONS], BUTTON_LEFT
	jz pressed_left
	test [nn+BUTTONS], BUTTON_RIGHT
	jz pressed_right
	test [nn+BUTTONS], BUTTON_UP
	jz pressed_up
	test [nn+BUTTONS], BUTTON_DOWN
	jz pressed_down
	
	jmp main_loop
	
pressed_left:
	mov a, [bearX]
	cmp a, 1
	jl main_loop ; would this put bear off-screen?
	call erase_current_tile_for_bear_movement
	dec a
	mov [bearX], a
	jmp main_loop
pressed_right:
	mov a, [bearX]
	cmp a, 10
	jg main_loop ; would this put bear off-screen?
	call erase_current_tile_for_bear_movement
	inc a
	mov [bearX], a
	jmp main_loop
pressed_up:
	mov a, [bearY]
	cmp a, 1
	jl main_loop ; would this put bear off-screen?
	call erase_current_tile_for_bear_movement
	dec a
	mov [bearY], a
	jmp main_loop
pressed_down:
	mov a, [bearY]
	cmp a, 6
	jg main_loop ; would this put bear off-screen?
	call erase_current_tile_for_bear_movement
	inc a
	mov [bearY], a
	jmp main_loop
	
;-----------------------------------------------------------------------------
; Erase the current tile in preparation for a bear move
;-----------------------------------------------------------------------------
erase_current_tile_for_bear_movement:
	push a
	push b
	mov x1, tile_empty
	mov a, [bearX]
	mov b, [bearY]
	call put_tile
	
	pop b
	pop a
	ret
	
;-----------------------------------------------------------------------------
; Fills screen with cookies
;-----------------------------------------------------------------------------
fill_screen:
	mov x1, tile_cookie
	mov x2, tilemap
	mov b, 12*8
fill_screen_loop:
	mov a, [x1]
	mov [x2+l], a
	inc l
	jdbnz fill_screen_loop
	ret

;-----------------------------------------------------------------------------
; Input:
; 		x1 - tile data
;		a - column
;		b - row
;-----------------------------------------------------------------------------
put_tile:
	push a
	push b
put_tile_loop:
	add a, 12      ; a := a + 12*b
	jdbnz put_tile_loop ;
	
	mov x2, tilemap
	mov l, a
	mov b, [x1]
	mov [x2+l], b
	
	pop b
	pop a
	ret

;-----------------------------------------------------------------------------
; keys_wait_for_press
;-----------------------------------------------------------------------------
keys_wait_for_press:
	mov		a, [nn+BUTTONS]
	cmp		a, 0xFF
	jz		keys_wait_for_press
	ret

;-----------------------------------------------------------------------------
; keys_wait_for_release
;-----------------------------------------------------------------------------
keys_wait_for_release:
	mov		a, [nn+BUTTONS]
	cmp		a, 0xFF
	jnz		keys_wait_for_release
	ret

	
;-----------------------------------------------------------------------------
; Data
;-----------------------------------------------------------------------------

tile_empty: .db 0
tile_bear: .db 1
tile_cookie: .db 2

author: .db "Written by Sebastian Mihai, 2015"

	.align 8

tile_data:
	.incbin includes/tiles.bin

	.end
